# Copyright 2011 Splunk, Inc.                                                                       
#                                                                                                        
#   Licensed under the Apache License, Version 2.0 (the "License");                                      
#   you may not use this file except in compliance with the License.                                     
#   You may obtain a copy of the License at                                                              
#                                                                                                        
#       http://www.apache.org/licenses/LICENSE-2.0                                                       
#                                                                                                        
#   Unless required by applicable law or agreed to in writing, software                                  
#   distributed under the License is distributed on an "AS IS" BASIS,                                    
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                             
#   See the License for the specific language governing permissions and                                  
#   limitations under the License.    

from distutils.version import LooseVersion
import logging
import os
import sys

import cherrypy

import splunk
import splunk.entity as entity
from splunk.util import normalizeBoolean as normBool 
import splunk.appserver.mrsparkle.controllers as controllers
import splunk.appserver.mrsparkle.lib.util as util

from splunk.appserver.mrsparkle.lib.decorators import expose_page
from splunk.appserver.mrsparkle.lib.routes import route
from splunk.models.app import App

logger = logging.getLogger('splunk.unix_setup')

dir = os.path.join(util.get_apps_dir(), __file__.split('.')[-2], 'bin')
if not dir in sys.path:
    sys.path.append(dir)

from unix.models.input import MonitorInput, ScriptedInput

class UnixSetup(controllers.BaseController):
    '''Unix Setup Controller'''
 
    @route('/:app/:action=setup')
    @expose_page(must_login=True, methods=['GET']) 
    def setup(self, app, action, **kwargs):
        ''' show the setup page '''

        host_app = cherrypy.request.path_info.split('/')[3]
        user = cherrypy.session['user']['name'] 

        mon = MonitorInput.all()
        mon = mon.filter_by_app(app)
  
        scripted = ScriptedInput.all()
        scripted = scripted.filter_by_app(app)
   
        return self.render_template('/%s:/templates/setup_show.html' % host_app, 
                                    dict(mon=mon, scripted=scripted, app=app))

    @route('/:app/:action=success')
    @expose_page(must_login=True, methods=['GET']) 
    def success(self, app, action, **kwargs):
        ''' render the success page '''

        host_app = cherrypy.request.path_info.split('/')[3]
        return self.render_template('/%s:/templates/setup_success.html' \
                                    % host_app,
                                    dict(app=app))

    @route('/:app/:action=failure')
    @expose_page(must_login=True, methods=['GET']) 
    def failure(self, app, action, **kwargs):
        ''' render the failure page '''

        host_app = cherrypy.request.path_info.split('/')[3]
        return self.render_template('/%s:/templates/setup_failure.html' \
                                    % host_app,
                                    dict(app=app))

    @route('/:app/:action=unauthorized')
    @expose_page(must_login=True, methods=['GET']) 
    def unauthorized(self, app, action, **kwargs):
        ''' render the unauthorized page '''

        host_app = cherrypy.request.path_info.split('/')[3]
        return self.render_template('/%s:/templates/setup_403.html' \
                                    % host_app,
                                    dict(app=app))

    @route('/:app/:action=save')
    @expose_page(must_login=True, methods=['POST']) 
    def save(self, app, action, **params):
        ''' save the posted setup content '''

        host_app = cherrypy.request.path_info.split('/')[3]
        user = cherrypy.session['user']['name'] 

        mon = MonitorInput.all()
        mon = mon.filter_by_app(app)
  
        scripted = ScriptedInput.all()
        scripted = scripted.filter_by_app(app)
            
        for m in mon:
            disabled = normBool(params.get(m.name + '.disabled'))
            if disabled:
                m.disable()
            else:
                m.enable()
            m.share_global()

        for s in scripted:
            disabled = normBool(params.get(s.name + '.disabled'))
            if disabled:
                s.disable()
            else:
                s.enable()
            s.share_global()
            interval = params.get(s.name + '.interval')
            if interval:
                s.interval = interval
                try:
                    if not s.passive_save():
                        logger.error(m.errors)
                        return self.render_template(
                            '/%s:/templates/setup_show.html' \
                                % host_app,
                            dict(app=app, 
                                 errors=s,
                                 scripted=scripted, 
                                 mon=mon)
                        )
                except splunk.AuthorizationFailed:
                    raise cherrypy.HTTPRedirect(self._redirect(host_app, app, 'unauthorized'))
                except:
                    raise cherrypy.HTTPRedirect(self._redirect(host_app, app, 'failure'))

        logger.debug('Splunk Version = %s' % self._get_version())
        if self._get_version() <= LooseVersion('4.2.2'):
            import splunk.bundle as bundle
            temp_app = bundle.getConf('app', namespace=host_app, owner='nobody') 
            temp_app['install']['is_configured'] = 'true'
        else:
            this_app = App.get(App.build_id(host_app, host_app, user))
            this_app.is_configured = True 
            this_app.passive_save()

        logger.info('%s - App setup successful' % host_app)

        raise cherrypy.HTTPRedirect(self._redirect(host_app, app, 'success'))

    def _redirect(self, host_app, app, endpoint):
        return self.make_url(['custom', host_app, 'unixsetup', app, endpoint])

    def _get_version(self):
        try:
            return LooseVersion(entity.getEntity('server/info', 'server-info')['version'])
        except:
            return LooseVersion('0.0')
